@@ -4,22 +4,23 @@
 
 """Checkers for various standard library functions."""
 
+from __future__ import annotations
+
 import sys
 from collections.abc import Iterable
-from typing import TYPE_CHECKING, Dict, List, Optional, Set, Tuple
+from typing import TYPE_CHECKING
 
 import astroid
 from astroid import nodes
 
 from pylint import interfaces
 from pylint.checkers import BaseChecker, DeprecatedMixin, utils
-from pylint.interfaces import IAstroidChecker
 
 if TYPE_CHECKING:
     from pylint.lint import PyLinter
 
 OPEN_FILES_MODE = ("open", "file")
-OPEN_FILES_ENCODING = ("open", "read_text", "write_text")
+OPEN_FILES_FUNCS = OPEN_FILES_MODE + ("read_text", "write_text")
 UNITTEST_CASE = "unittest.case"
 THREADING_THREAD = "threading.Thread"
 COPY_COPY = "copy.copy"
@@ -37,17 +38,7 @@ LRU_CACHE = {
 NON_INSTANCE_METHODS = {"builtins.staticmethod", "builtins.classmethod"}
 
 
-DEPRECATED_MODULES = {
-    (0, 0, 0): {"tkinter.tix", "fpectl"},
-    (3, 2, 0): {"optparse"},
-    (3, 3, 0): {"xml.etree.cElementTree"},
-    (3, 4, 0): {"imp"},
-    (3, 5, 0): {"formatter"},
-    (3, 6, 0): {"asynchat", "asyncore"},
-    (3, 7, 0): {"macpath"},
-    (3, 9, 0): {"lib2to3", "parser", "symbol", "binhex"},
-    (3, 10, 0): {"distutils"},
-}
+# For modules, see ImportsChecker
 
 DEPRECATED_ARGUMENTS = {
     (0, 0, 0): {
@@ -98,7 +89,7 @@ DEPRECATED_DECORATORS = {
 }
 
 
-DEPRECATED_METHODS: Dict = {
+DEPRECATED_METHODS: dict = {
     0: {
         "cgi.parse_qs",
         "cgi.parse_qsl",
@@ -153,6 +144,7 @@ DEPRECATED_METHODS: Dict = {
             "ntpath.splitunc",
             "os.path.splitunc",
             "os.stat_float_times",
+            "turtle.RawTurtle.settiltangle",
         },
         (3, 2, 0): {
             "cgi.escape",
@@ -241,11 +233,23 @@ DEPRECATED_METHODS: Dict = {
             "threading.Thread.setDaemon",
             "cgi.log",
         },
+        (3, 11, 0): {
+            "locale.getdefaultlocale",
+            "unittest.TestLoader.findTestCases",
+            "unittest.TestLoader.loadTestsFromTestCase",
+            "unittest.TestLoader.getTestCaseNames",
+        },
     },
 }
 
 
 DEPRECATED_CLASSES = {
+    (3, 2, 0): {
+        "configparser": {
+            "LegacyInterpolation",
+            "SafeConfigParser",
+        },
+    },
     (3, 3, 0): {
         "importlib.abc": {
             "Finder",
@@ -287,6 +291,11 @@ DEPRECATED_CLASSES = {
             "MailmanProxy",
         }
     },
+    (3, 11, 0): {
+        "webbrowser": {
+            "MacOSX",
+        },
+    },
 }
 
 
@@ -321,10 +330,10 @@ def _check_mode_str(mode):
 
 
 class StdlibChecker(DeprecatedMixin, BaseChecker):
-    __implements__ = (IAstroidChecker,)
     name = "stdlib"
 
     msgs = {
+        **{k: v for k, v in DeprecatedMixin.msgs.items() if k[1:3] == "15"},
         "W1501": (
             '"%s" is not a valid mode for open.',
             "bad-open-mode",
@@ -349,13 +358,6 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
             "condition will be always true. In this case a warning "
             "should be emitted.",
         ),
-        "W1505": (
-            "Using deprecated method %s()",
-            "deprecated-method",
-            "The method is marked as deprecated and will be removed in "
-            "a future version of Python. Consider looking for an "
-            "alternative in the documentation.",
-        ),
         "W1506": (
             "threading.Thread needs the target function",
             "bad-thread-instantiation",
@@ -400,27 +402,12 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
             "`check` keyword to make clear what the error-handling behavior is."
             "https://docs.python.org/3/library/subprocess.html#subprocess.run",
         ),
-        "W1511": (
-            "Using deprecated argument %s of method %s()",
-            "deprecated-argument",
-            "The argument is marked as deprecated and will be removed in the future.",
-        ),
-        "W1512": (
-            "Using deprecated class %s of module %s",
-            "deprecated-class",
-            "The class is marked as deprecated and will be removed in the future.",
-        ),
-        "W1513": (
-            "Using deprecated decorator %s()",
-            "deprecated-decorator",
-            "The decorator is marked as deprecated and will be removed in the future.",
-        ),
         "W1514": (
             "Using open without explicitly specifying an encoding",
             "unspecified-encoding",
             "It is better to specify an encoding when opening documents. "
             "Using the system default implicitly can create problems on other operating systems. "
-            "See https://www.python.org/dev/peps/pep-0597/",
+            "See https://peps.python.org/pep-0597/",
         ),
         "W1515": (
             "Leaving functions creating breakpoints in production code is not recommended",
@@ -428,27 +415,29 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
             "Calls to breakpoint(), sys.breakpointhook() and pdb.set_trace() should be removed "
             "from code that is not actively being debugged.",
         ),
-        "W1517": (
-            "'lru_cache(maxsize=None)' will keep all method args alive indefinitely, including 'self'",
-            "cache-max-size-none",
-            "By decorating a method with lru_cache the 'self' argument will be linked to "
-            "the lru_cache function and therefore never garbage collected. Unless your instance "
+        "W1518": (
+            "'lru_cache(maxsize=None)' or 'cache' will keep all method args alive indefinitely, including 'self'",
+            "method-cache-max-size-none",
+            "By decorating a method with lru_cache or cache the 'self' argument will be linked to "
+            "the function and therefore never garbage collected. Unless your instance "
             "will never need to be garbage collected (singleton) it is recommended to refactor "
             "code to avoid this pattern or add a maxsize to the cache."
             "The default value for maxsize is 128.",
-            {"old_names": [("W1516", "lru-cache-decorating-method")]},
+            {
+                "old_names": [
+                    ("W1516", "lru-cache-decorating-method"),
+                    ("W1517", "cache-max-size-none"),
+                ]
+            },
         ),
     }
 
-    def __init__(self, linter: Optional["PyLinter"] = None) -> None:
+    def __init__(self, linter: PyLinter) -> None:
         BaseChecker.__init__(self, linter)
-        self._deprecated_methods: Set[str] = set()
-        self._deprecated_arguments: Dict[
-            str, Tuple[Tuple[Optional[int], str], ...]
-        ] = {}
-        self._deprecated_classes: Dict[str, Set[str]] = {}
-        self._deprecated_modules: Set[str] = set()
-        self._deprecated_decorators: Set[str] = set()
+        self._deprecated_methods: set[str] = set()
+        self._deprecated_arguments: dict[str, tuple[tuple[int | None, str], ...]] = {}
+        self._deprecated_classes: dict[str, set[str]] = {}
+        self._deprecated_decorators: set[str] = set()
 
         for since_vers, func_list in DEPRECATED_METHODS[sys.version_info[0]].items():
             if since_vers <= sys.version_info:
@@ -459,12 +448,11 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
         for since_vers, class_list in DEPRECATED_CLASSES.items():
             if since_vers <= sys.version_info:
                 self._deprecated_classes.update(class_list)
-        for since_vers, mod_list in DEPRECATED_MODULES.items():
-            if since_vers <= sys.version_info:
-                self._deprecated_modules.update(mod_list)
         for since_vers, decorator_list in DEPRECATED_DECORATORS.items():
             if since_vers <= sys.version_info:
                 self._deprecated_decorators.update(decorator_list)
+        # Modules are checked by the ImportsChecker, because the list is
+        # synced with the config argument deprecated-modules
 
     def _check_bad_thread_instantiation(self, node):
         if not node.kwargs and not node.keywords and len(node.args) <= 1:
@@ -492,7 +480,7 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
                 self.add_message("shallow-copy-environ", node=node)
                 break
 
-    @utils.check_messages(
+    @utils.only_required_for_messages(
         "bad-open-mode",
         "redundant-unittest-assert",
         "deprecated-method",
@@ -514,18 +502,13 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
             if inferred is astroid.Uninferable:
                 continue
             if inferred.root().name in OPEN_MODULE:
-                if (
-                    isinstance(node.func, nodes.Name)
-                    and node.func.name in OPEN_FILES_MODE
-                ):
-                    self._check_open_mode(node)
-                if (
-                    isinstance(node.func, nodes.Name)
-                    and node.func.name in OPEN_FILES_ENCODING
-                    or isinstance(node.func, nodes.Attribute)
-                    and node.func.attrname in OPEN_FILES_ENCODING
-                ):
-                    self._check_open_encoded(node, inferred.root().name)
+                open_func_name: str | None = None
+                if isinstance(node.func, nodes.Name):
+                    open_func_name = node.func.name
+                if isinstance(node.func, nodes.Attribute):
+                    open_func_name = node.func.attrname
+                if open_func_name in OPEN_FILES_FUNCS:
+                    self._check_open_call(node, inferred.root().name, open_func_name)
             elif inferred.root().name == UNITTEST_CASE:
                 self._check_redundant_assert(node, inferred)
             elif isinstance(inferred, nodes.ClassDef):
@@ -545,58 +528,62 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
                     self.add_message("forgotten-debug-statement", node=node)
             self.check_deprecated_method(node, inferred)
 
-    @utils.check_messages("boolean-datetime")
+    @utils.only_required_for_messages("boolean-datetime")
     def visit_unaryop(self, node: nodes.UnaryOp) -> None:
         if node.op == "not":
             self._check_datetime(node.operand)
 
-    @utils.check_messages("boolean-datetime")
+    @utils.only_required_for_messages("boolean-datetime")
     def visit_if(self, node: nodes.If) -> None:
         self._check_datetime(node.test)
 
-    @utils.check_messages("boolean-datetime")
+    @utils.only_required_for_messages("boolean-datetime")
     def visit_ifexp(self, node: nodes.IfExp) -> None:
         self._check_datetime(node.test)
 
-    @utils.check_messages("boolean-datetime")
+    @utils.only_required_for_messages("boolean-datetime")
     def visit_boolop(self, node: nodes.BoolOp) -> None:
         for value in node.values:
             self._check_datetime(value)
 
-    @utils.check_messages("cache-max-size-none")
+    @utils.only_required_for_messages("method-cache-max-size-none")
     def visit_functiondef(self, node: nodes.FunctionDef) -> None:
         if node.decorators and isinstance(node.parent, nodes.ClassDef):
             self._check_lru_cache_decorators(node.decorators)
 
     def _check_lru_cache_decorators(self, decorators: nodes.Decorators) -> None:
         """Check if instance methods are decorated with functools.lru_cache."""
-        lru_cache_nodes: List[nodes.NodeNG] = []
+        lru_cache_nodes: list[nodes.NodeNG] = []
         for d_node in decorators.nodes:
             try:
                 for infered_node in d_node.infer():
                     q_name = infered_node.qname()
-                    if q_name in NON_INSTANCE_METHODS or q_name not in LRU_CACHE:
+                    if q_name in NON_INSTANCE_METHODS:
                         return
 
                     # Check if there is a maxsize argument set to None in the call
-                    if isinstance(d_node, nodes.Call):
+                    if q_name in LRU_CACHE and isinstance(d_node, nodes.Call):
                         try:
                             arg = utils.get_argument_from_call(
                                 d_node, position=0, keyword="maxsize"
                             )
                         except utils.NoSuchArgumentError:
-                            return
+                            break
 
                         if not isinstance(arg, nodes.Const) or arg.value is not None:
-                            return
+                            break
 
-                    lru_cache_nodes.append(d_node)
-                    break
+                        lru_cache_nodes.append(d_node)
+                        break
+
+                    if q_name == "functools.cache":
+                        lru_cache_nodes.append(d_node)
+                        break
             except astroid.InferenceError:
                 pass
         for lru_cache_node in lru_cache_nodes:
             self.add_message(
-                "cache-max-size-none",
+                "method-cache-max-size-none",
                 node=lru_cache_node,
                 confidence=interfaces.INFERENCE,
             )
@@ -626,25 +613,10 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
         ):
             self.add_message("boolean-datetime", node=node)
 
-    def _check_open_mode(self, node):
-        """Check that the mode argument of an open or file call is valid."""
-        try:
-            mode_arg = utils.get_argument_from_call(node, position=1, keyword="mode")
-        except utils.NoSuchArgumentError:
-            return
-        if mode_arg:
-            mode_arg = utils.safe_infer(mode_arg)
-            if isinstance(mode_arg, nodes.Const) and not _check_mode_str(
-                mode_arg.value
-            ):
-                self.add_message(
-                    "bad-open-mode",
-                    node=node,
-                    args=mode_arg.value or str(mode_arg.value),
-                )
-
-    def _check_open_encoded(self, node: nodes.Call, open_module: str) -> None:
-        """Check that the encoded argument of an open call is valid."""
+    def _check_open_call(
+        self, node: nodes.Call, open_module: str, func_name: str
+    ) -> None:
+        """Various checks for an open call."""
         mode_arg = None
         try:
             if open_module == "_io":
@@ -661,6 +633,17 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
         if mode_arg:
             mode_arg = utils.safe_infer(mode_arg)
 
+            if (
+                func_name in OPEN_FILES_MODE
+                and isinstance(mode_arg, nodes.Const)
+                and not _check_mode_str(mode_arg.value)
+            ):
+                self.add_message(
+                    "bad-open-mode",
+                    node=node,
+                    args=mode_arg.value or str(mode_arg.value),
+                )
+
         if (
             not mode_arg
             or isinstance(mode_arg, nodes.Const)
@@ -749,10 +732,6 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
         else:
             self.add_message(message, node=node, args=(name, call_arg.pytype()))
 
-    def deprecated_modules(self):
-        """Callback returning the deprecated modules."""
-        return self._deprecated_modules
-
     def deprecated_methods(self):
         return self._deprecated_methods
 
@@ -766,5 +745,5 @@ class StdlibChecker(DeprecatedMixin, BaseChecker):
         return self._deprecated_decorators
 
 
-def register(linter: "PyLinter") -> None:
+def register(linter: PyLinter) -> None:
     linter.register_checker(StdlibChecker(linter))
